%Function multilev_new is a multi-level multi-harmonic signal design algorithm 
%based on the time-frequency swapping method. The quantizer parameter is 
%chosen based on the optimal signal produced when Schroeder phases are applied. 
%The algorithm uses a combination of both Schroeder phases and random phases as 
%initial values during different trials. 
%
%function signal = multilev_new(N,freqv,ampv,no_f,nolev,option,rnst,q)
%
%Input Arguments
%N ------ Length of signal
%freqv -- Harmonic vector
%ampv  -- Absolute amplitudes at specified harmonics
%no_f  -- Harmonic(s) and its multiples to be suppressed
%nolev -- Desired number of signal levels
%option - 1 if effect of ZOH is to be taken into account, 0 if otherwise
%         default = 1
%rnst --- Number of starts of time-frequency routine, default = 10000
%q ------ Quantizer parameter, default = [1:0.1:4]
%
%Output Argument
%signal - Designed signal
%
%Example without harmonic suppression:
%
%signal = multilev_new(256,[1:10],ones(10,1),0,3,1,100,[1:.1:4]);
%							
%will produce a signal with N=256, with a near-uniform amplitude spectrum in 
%harmonics 1 to 10, after passing through ZOH. The algorithm will make 
%length(q) = 31 trials with Schroeder phases to select the optimum quantizer, 
%and then make rnst-length(q) = 69 trials using this quantizer with random 
%initial phases.
%Example with harmonic suppression:
%
%freqv=[1,5,7,11,13,17,19];
%ampv=ones(size(f));
%no_f=[2,3];
%signal = multilev_new(300,freqv,ampv,no_f,3);
%
%will produce a signal of length 300 (a factor of 2x3) with all multiples of 
%harmonics 2 and 3 suppressed. N MUST be a factor of each element of no_f. Other 
%possibilities (but less useful) for no_f are [2,5],5,[2,7],7,[2,11],.... In 
%this example, option, rnst and q are set to default settings.
%
%   Author: A.H. Tan 
%   Faculty of Engineering, Multimedia University, Malaysia
%   12 January 2004 
%   Modified from 'multilev' by A.S. McCormack (University of Warwick)

function signal = multilev_new(N,freqv,ampv,no_f,nolev,option,rnst,q)

%default settings
if nargin<8, q=[1:0.1:4]; end
if nargin<7, rnst = 10000; end
if nargin<6, option = 1; end
low=0;,lowest=0;,divmax=q(1);,maxiter=20;fail=0;
ampv=ampv(:);,freqv=freqv(:);best=zeros(N,1);

h = waitbar(0,'Please Wait...');
for pnt=1:rnst
	a=zeros(max(freqv),1);
	a(freqv)=abs(ampv);
	fno=length(a);
	ph=zeros(size(a));
	if pnt <= length(q)
		an=a./(ones(fno,1)*sqrt(sum(a.^2)));an=2*pi*an.^2;
		for i=2:length(freqv)
      	ph(freqv(i))=ph(freqv(i-1))-sum(an(freqv(1:i-1)));		%Schroeder phases
		end
		div=q(pnt);
	else
		ph=2*pi*randn(size(a));ph1=ph(freqv);div=divmax;			%random phases
	end
   Xf=(a.*exp(j*ph));
   if option == 1
      Xf=Xf(:).*((pi*[1:max(freqv)]/N)./(sin(pi*[1:max(freqv)]/N)))';  
   else
      Xf=Xf(:);
   end
   xtim=N*real(ifft([0;Xf],N));xtim=xtim(:);
	low=0;,lev=1;
	if mod(nolev,2) == 0 											%even number of levels
		levels=fliplr(1:(-1/(nolev/2)):1/(nolev/2));
		levels=[-fliplr(levels) levels];
		for i=1:(nolev/2)
			lev(i+1)=lev(i)/div;
		end
		lev=[lev fliplr(lev(1:(length(lev)-1)))];	
	else																	%odd number of levels
		levels=-1:(1/((nolev-1)/2)):1;
		for i=1:ceil(nolev/2)-1
			lev(i+1)=lev(i)/div;
		end
		lev=[lev fliplr(lev)];
	end
   decgen=[-1:1/(nolev/2):1];dec=decgen*max(abs(xtim)).*lev;
   %quantization
	for i=1:N
		for sec=1:nolev
			if (dec(sec) <= xtim(i)) & (xtim(i) <= dec(sec+1))
				sig(i,1)=levels(sec);
			end
		end
	end 
   ftim=fft(xtim)/N;ab=abs(ftim);ab=ab(:);
   fsig=fft(sig)/N;asig=abs(fsig);
   th=angle(fsig(freqv+1));th_old=zeros(size(th));
   %time-frequency swapping
	for k=1:maxiter
		ang=angle(fsig);cx=p2r(ab,ang*180/pi);
		xtim=real(ifft(cx));dec=decgen*max(abs(xtim)).*lev;
		for i=1:N
        	for sec=1:nolev
          	if (dec(sec) <= xtim(i)) & (xtim(i) <= dec(sec+1))
            	sig(i)=levels(sec);
            end
        	end
		end
		if any(no_f~=0)
			l_nof=length(no_f);
         sp=N/(l_nof*no_f(l_nof));sw=(N/2);inx=N/(no_f(l_nof));st=zeros(sp,1);
         for i1=0:no_f(l_nof)-1
				for i=1:sp
					st(i)=st(i)+sig(i+(i1*inx));
				end
         end
			for i=1:sp;
				if st(i)~=0
					for tst=0:no_f(l_nof)-1
						inx1=i+(tst*inx);tstsig=sig(inx1)-((1^(tst+1))*st(i));
						if abs(tstsig) <= 1
							sig(inx1)=tstsig;
							if l_nof == 2
								inx2=inx1+sw;
                     	if inx2 > N
                        	inx2=rem(inx1+sw,N);
                     	end
								sig(inx2)=-tstsig;
							end
							break;
						end
					end
					if abs(tstsig) > 1
						fail=1;
						break;
					end
				end
			end
		end
      fsig=fft(sig)/N;asig=abs(fsig);
      %measure of signal quality to be maximised
      EMINE = calculate_EMINE_new(sig,freqv,ampv,option);
      if EMINE == 0
          EMINE = 0.000001;
      end 
      if option == 1   
         measure=1000-10^8/calculate_PIPSE(sig,freqv)^2/EMINE^2/2;
      else
         DFT = abs(fft(sig));DFT(1)=[];DFT=DFT(freqv);DFT_squared = 2*DFT.^2;
         PIPSE = 200*sqrt(sum(DFT_squared))/length(sig)/(max(sig)-min(sig));
         measure=1000-10^8/PIPSE^2/EMINE^2/2;
      end   
      if (measure > low & fail~=1)
         low=measure;best=sig;
      end
	  th_old=th;th=angle(fsig(freqv+1));
	  if all(abs(th_old-th)>0.00001)~=1
		  break;
	  end
	  fail=0;
	end
	em(pnt)=low;
	if low >= lowest	
		if pnt <= length(q)
         divmax=div;
		end
		lowest=low;signal=best;
	end
waitbar(pnt/rnst,h);
end
close(h)

%display error message
if low == 0
	fprintf(1,'Sorry, algorithm could not find signal with desired spectrum.\n')
	fprintf('Try different q and/or larger number of trials.\n')
	signal=[];
end